from websockets.sync.server import serve, ServerConnection, Server
from websockets.exceptions import ConnectionClosedOK
import requests
import json

from .project import Project
from .backend import uploadOrder

import logging
logger = logging.getLogger(__name__)

def listen(host: str, port: int, callback) -> Server:
    '''
    Start a WebSockets server on the specified host and port.

    Parameters
    ----------
    host : str
        The hostname or IP address to bind the server to.
    port : int
        The port number to bind the server to.
    callback : callable
        A function that will be called for each new connection. It should take a `ServerConnection` object as its only argument.

    Returns
    -------
    Server : websockets.sync.server.Server

    Notes
    -----
    This function does not block; it returns immediately after starting the server.

    See Also
    --------
    websockets.sync.server.serve : The underlying function used to start the server.
    '''
    usePort = port
    while usePort - port <= 4:
        try:
            return serve(callback, host, usePort)
        except OSError as error:
            logger.warning(f'Failed to start server ({host}:{usePort}), try using next port number: %s', error)
        usePort += 1

    raise OSError(f'Failed to start server ({host}:{port}) as all port numbers in range ({port}-{port+5}) are already in use')

def handleConnection(websocket: ServerConnection, project: Project, host: str, thread) -> None:
    '''
    Handles a new connection to the WebSockets server.

    Parameters
    ----------
    websocket : ServerConnection
        The newly established connection.
    project : Project
        The current project being worked on.
    host : str
        The hostname or IP address of the server.
    thread :
        A thread object that manages the WebSocket server.


    Returns
    -------
    None

    Notes
    -----
    This function is called for each new connection to the server.
    It handles authentication, sends the project data, and updates the order when a new one is submitted.
    Finally, it confirms the new order with the client.


    See Also
    --------
    websockets.sync.server.serve : The underlying function used to start the server.
    '''
    try:
        sendProject(websocket, project)

        token = getAuthentificationToken(websocket)
        try:
            modifiedOrder = awaitNewOrder(websocket)
        except ConnectionClosedOK:
            # user closed the page
            return

        project.update(modifiedOrder)

        try:
            uploadOrder(project, host, token)
        except requests.exceptions.HTTPError as error:
            match error.response.status_code:
                case 400:
                    logger.error(error.response.content)
                    
            raise

        confirmNewOrder(websocket)
        thread.closeWebsocketServer()
    except:
        sendErrorMessage(websocket)
        thread.closeWebsocketServer()
        raise

def getAuthentificationToken(websocket: ServerConnection) -> str:
    """
    Retrieves an authentication token from the client.

    Parameters
    ----------
    websocket : ServerConnection
        The WebSocket connection object.

    Returns
    -------
    str
        The authentication token.
    """
    websocket.send(json.dumps({'type': 'getToken'}))
    response = websocket.recv()
    data = json.loads(response)
    if data['type'] == 'token':
        return data['token']

def sendProject(websocket: ServerConnection, project: Project) -> None:
    """
    Sends project information to the client.

    Parameters
    ----------
    websocket : ServerConnection
        The WebSocket connection object.
    project : Project
        The project object.
    """
    message = {
        'type': 'projectInformation',
        'untrackedElements': [{
            'type': None,
            'material': None,
            'color': None,
            'height': 0,
            'comment': '',
            'file': path.name,
            'teeth': [],
            'amenities': []
        } for path in project.untrackedFiles],
        'sceneFilePresent': project.sceneFilePresent,
        'order': project.order
    }
    websocket.send(json.dumps(message))

def awaitNewOrder(websocket: ServerConnection) -> object:
    """
    Waits for a new order message from the client and returns the order data.

    Parameters
    ----------
    websocket : ServerConnection
        The WebSocket connection object.

    Returns
    -------
    object
        The order data.
    """
    confirmed = False
    while not confirmed:
        request = websocket.recv()
        data = json.loads(request)
        if data['type'] != 'createOrder':
            continue
        
        return data['order']

def confirmNewOrder(websocket: ServerConnection) -> None:
    """
    Confirms a new order with the client.

    Parameters
    ----------
    websocket : ServerConnection
        The WebSocket connection object.
    """
    websocket.send(json.dumps({
        'type': 'confirmNewOrder'
    }))

def sendErrorMessage(websocket: ServerConnection) -> None:
    websocket.send(json.dumps({
        'type': 'error'
    }))


import argparse
import sys
import webbrowser
import asyncio
from pathlib import Path
import pkg_resources

import logging
logger = logging.getLogger(__name__)

from .project import Project
from .updater import newVersionAvailable, update
from .server import listen, handleConnection

async def run(projectPath: Path, host: str, uploadURL: str):
    logger.info(f'Starting millConnect client {pkg_resources.get_distribution('millconnect').version}.')

    # TODO: add check if application is currently running.
    # TODO: Could intercept with restarting into new application.

    if newVersionAvailable():
        update()

    project = Project(projectPath)

    stop = asyncio.get_running_loop().create_future()

    # create socket for frontend connection
    server = await listen('localhost', 8765, lambda socket: handleConnection(socket, project, host, stop))

    webbrowser.open_new_tab(uploadURL)

    await stop
    server.close()
    await server.wait_closed()

def main():
    logging.basicConfig(stream=sys.stdout, level=logging.INFO, format='%(asctime)s %(name)s %(levelname)s: %(message)s')

    parser = argparse.ArgumentParser(description='millConnect order creator.')
    parser.add_argument('projectPath', type=Path, help='The path to a project folder.')
    parser.add_argument('--host', nargs='?', type=str, default='https://api.connect.millhouse.de/v1', help='The URL of the api.')
    parser.add_argument('--uploadURL', nargs='?', type=str, default='https://connect.millhouse.de/uploadOrder', help='The URL to open in the browser.')

    args = parser.parse_args()
    asyncio.run(run(projectPath=args.projectPath, host=args.host, uploadURL=args.uploadURL))

if __name__ == '__main__':
    main()


from typing import Any
import argparse
import sys
import webbrowser
from pathlib import Path
from importlib import metadata

import logging
logger = logging.getLogger('millconnect.gui')

from PyQt6.QtCore import QTimer ,QRunnable, QThreadPool, pyqtSlot, pyqtSignal, QTranslator, QDir, QStandardPaths, Qt
from PyQt6.QtWidgets import QApplication, QLabel, QWidget, QVBoxLayout, QProgressBar, QStackedWidget, QFileDialog, QPushButton
from PyQt6.QtGui import QIcon

from .project import Project
from .updater import newVersionAvailable, update
from .server import listen, handleConnection

class ProjectSelector(QWidget):
    folderSelected = pyqtSignal(Path)
    canceledSelection = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)

        self.projectFolder: Path = None

        layout = QVBoxLayout()
        self.setLayout(layout)

        self.label = QLabel(self, text=self.tr('Select a project folder.'))
        layout.addWidget(self.label)

    def showEvent(self, a0):
        super().showEvent(a0)
        QTimer.singleShot(0, self.selectFolder)

    def selectFolder(self):
        defaultFolder = QDir.toNativeSeparators(QStandardPaths.writableLocation(QStandardPaths.StandardLocation.DesktopLocation))
        self.projectFolder = QFileDialog.getExistingDirectory(self, self.tr('Open project folder'), defaultFolder)
        logger.debug(f'selected folder: "{self.projectFolder}".')

        if not self.projectFolder:
            self.canceledSelection.emit()
        else:
            self.folderSelected.emit(Path(self.projectFolder))

class UpdateRunner(QRunnable):

    def __init__(self, parent=None):
        super().__init__()
        self.parent = parent

    def run(self):
        self.checkForUpdate()

    def checkForUpdate(self):
        logger.info('Checking for update.')
        if newVersionAvailable():
            logger.info('New version available.')
            self.parent.downloadInitialiated()
            update()
            self.parent.restartSignal.emit()
        else:
            logger.info('No new version available.')
            self.parent.noUpdate()
            self.parent.noUpdateSignal.emit()

class UpdatingDialog(QWidget):
    noUpdateSignal = pyqtSignal()
    restartSignal = pyqtSignal()

    def __init__(self, parent=None, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)

        layout = QVBoxLayout()
        self.setLayout(layout)

        self.label = QLabel(self, text=self.tr('Checking for updates.'))
        layout.addWidget(self.label)

        self.progressBar = QProgressBar(self, minimum=0, maximum=100, value=0)
        layout.addWidget(self.progressBar)


    @pyqtSlot()
    def initializeUpdate(self):
        self.threadpool = QThreadPool()
        self.threadpool.start(UpdateRunner(self))

    @pyqtSlot()
    def downloadInitialiated(self):
        self.label.setText(self.tr('Downloading...'))
        self.progressBar.setValue(50)
        self.update()

    @pyqtSlot()
    def restartInitiated(self):
        self.label.setText(self.tr('Update successfull. Restarting...'))
        self.progressBar.setValue(100)
        self.update()

    @pyqtSlot()
    def noUpdate(self):
        self.label.setText(self.tr('Application is up-to-date.'))
        self.progressBar.setValue(100)
        self.update()

class WebsocketServer(QRunnable):

    def __init__(self, parent, callback):
        super().__init__()
        self.parent = parent
        self.server = None
        self.callback = callback

    @pyqtSlot()
    def run(self):
        logger.info('Starting websocket server.')
        try:
            self.server = listen('localhost', 8765, self.callback)
            self.server.serve_forever()
        except OSError:
            logger.error('Could not start websocket server.')
            self.parent.closeWebsocketServer(False)

    def stopServer(self):
        logger.info('Stopping websocket server.')
        if self.server:
            self.server.shutdown()

class BrowserDialog(QWidget):
    uploadFinished = pyqtSignal()
    uploadFailed = pyqtSignal()

    def __init__(self, parent=None, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)

        self.worker: WebsocketServer = None

        layout = QVBoxLayout()
        self.setLayout(layout)

        self.label = QLabel(self, text=self.tr('Please follow the instructions in the browser.'))
        layout.addWidget(self.label)

        self.threadpool = QThreadPool()

    @pyqtSlot(str, int, Any)
    def createWebsocketServer(self, apiHost: str, project: Project):
        self.worker = WebsocketServer(self, lambda websocket: handleConnection(websocket, project, apiHost, self))
        self.threadpool.start(self.worker)

    def closeWebsocketServer(self, success=True):
        if self.worker:
            self.worker.stopServer()
        if success:
            self.uploadFinished.emit()
        else:
            self.uploadFailed.emit()

class ErrorDialog(QWidget):
    buttonPressed = pyqtSignal()

    def __init__(self, parent=None, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)

        layout = QVBoxLayout()
        self.setLayout(layout)

        self.label = QLabel(self, text=self.tr('An Error occurt during uploading of the project. Please contact the support.'))
        layout.addWidget(self.label)

        self.button = QPushButton(self.tr('OK'), self)
        layout.addWidget(self.button)

        self.button.clicked.connect(self.handleButtonPressed)

    @pyqtSlot()
    def handleButtonPressed(self):
        self.buttonPressed.emit()

class MainWindow(QWidget):
    def __init__(self, parent=None, projectPath: Path=None, host: str=None, uploadURL: str=None, no_update: bool=False, *args, **kwargs):
        super().__init__(parent, *args, **kwargs)

        self.projectPath = projectPath
        self.host = host
        self.uploadURL = uploadURL

        self.resize(600, 60)
        filePath = Path(__file__).parent / 'assets' / 'millhouse.png'
        self.setWindowIcon(QIcon(str(filePath)))

        layout = QVBoxLayout()
        self.setLayout(layout)

        self.stackedWidget = QStackedWidget(self)
        layout.addWidget(self.stackedWidget)
        
        self.updatingDialog = UpdatingDialog(self)
        self.stackedWidget.addWidget(self.updatingDialog)

        self.browserDialog = BrowserDialog(self)
        self.stackedWidget.addWidget(self.browserDialog)

        self.projectSelector = ProjectSelector(self)
        self.stackedWidget.addWidget(self.projectSelector)

        self.errorDialog = ErrorDialog(self)
        self.stackedWidget.addWidget(self.errorDialog)

        self.updatingDialog.restartSignal.connect(self.close)
        self.errorDialog.buttonPressed.connect(self.close)

        if self.projectPath:
            self.updatingDialog.noUpdateSignal.connect(self.showBrowserDialog)
            self.updatingDialog.noUpdateSignal.connect(self.startWebsocket)
        else:
            self.updatingDialog.noUpdateSignal.connect(self.showProjectSelector)
            self.projectSelector.canceledSelection.connect(self.close)
            self.projectSelector.folderSelected.connect(self.setProjectPath)

        self.browserDialog.uploadFinished.connect(self.close)
        self.browserDialog.uploadFailed.connect(self.showErrorDialog)

        if no_update:
            if self.projectPath:
                self.stackedWidget.setCurrentWidget(self.browserDialog)
            else:
                self.stackedWidget.setCurrentWidget(self.projectSelector)

            self.updatingDialog.noUpdateSignal.emit()
        else:
            self.stackedWidget.setCurrentWidget(self.updatingDialog)
            self.updatingDialog.initializeUpdate()

    @pyqtSlot()
    def showBrowserDialog(self):
        logger.info(f'Opening browser with URL: {self.uploadURL}.')
        webbrowser.open_new_tab(self.uploadURL)

        self.stackedWidget.setCurrentWidget(self.browserDialog)
        self.update()

    @pyqtSlot()
    def startWebsocket(self):
        self.project = Project(self.projectPath)
        self.browserDialog.createWebsocketServer(self.host, self.project)

    @pyqtSlot()
    def showProjectSelector(self):
        self.stackedWidget.setCurrentWidget(self.projectSelector)
        self.update()

    @pyqtSlot(Path)
    def setProjectPath(self, projectPath: Path):
        self.projectPath = projectPath
        self.showBrowserDialog()
        self.startWebsocket()

    @pyqtSlot()
    def showErrorDialog(self):
        self.stackedWidget.setCurrentWidget(self.errorDialog)
        self.update()
        self.setWindowState( (self.windowState() & ~Qt.WindowState.WindowMinimized) | Qt.WindowState.WindowActive)
        self.raise_()  # for MacOS
        self.activateWindow() # for Windows

    def closeEvent(self, event):
        self.browserDialog.closeWebsocketServer()
        return super().closeEvent(event)

def handle_exception(exc_type, exc_value, exc_traceback):
    logger.error("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))

    sys.__excepthook__(exc_type, exc_value, exc_traceback)
    return

sys.excepthook = handle_exception

def main():
    parser = argparse.ArgumentParser(description='millConnect order creator.')
    parser.add_argument('projectPath', nargs='?', type=Path, default=None, help='The path to a project folder.')
    parser.add_argument('--host', nargs='?', type=str, default='https://api.connect.millhouse.de/v1', help='The URL of the api, default: https://api.connect.millhouse.de/v1')
    parser.add_argument('--uploadURL', nargs='?', type=str, default='https://connect.millhouse.de/uploadOrder', help='The URL to open in the browser, default: https://connect.millhouse.de/uploadOrder')
    parser.add_argument('--no-update', action='store_true', help='Do not check for updates.')
    parser.add_argument('-v','--verbose', action='store_true', help='Verbose logging.')
    args = parser.parse_args()

    logFormatter = logging.Formatter('%(asctime)s %(name)s %(levelname)s: %(message)s')

    logPath = QDir.home()
    fileHandler = logging.FileHandler(logPath.filePath("millconnect-client.log"), mode='w')
    fileHandler.setFormatter(logFormatter)

    consoleHandler = logging.StreamHandler()
    consoleHandler.setFormatter(logFormatter)

    logLevel = logging.DEBUG if args.verbose else logging.INFO
    logger.setLevel(logLevel)

    logging.basicConfig(level=logLevel, handlers=[fileHandler, consoleHandler])

    app = QApplication(sys.argv)
    app.setOrganizationName('millhouse GmbH')
    app.setApplicationName('millCONNECT Client')

    if args.projectPath and args.projectPath.is_file():
        args.projectPath = args.projectPath.parent

    try:
        my_version = metadata.version('millconnect')
        app.setApplicationVersion(my_version)
        logger.info(f'Starting millConnect client {my_version}.')
    except metadata.PackageNotFoundError:
        logger.info('Running from source code. Disabled update check.')
        args.no_update = True

    translator = QTranslator()
    filePath = Path(__file__).parent / 'assets'
    if translator.load('de_DE', str(filePath)):
        app.installTranslator(translator)

    mainWindow = MainWindow(projectPath=args.projectPath, host=args.host, uploadURL=args.uploadURL, no_update=args.no_update)
    mainWindow.show()

    ret = app.exec()
    sys.exit(ret)

if __name__ == '__main__':
    main()



    


import unittest

from pathlib import Path
import tempfile

from src.millconnect.project import Project

class ExocadProject(unittest.TestCase):

    projectPath = Path('samples/exocad/02_Krone/156094_krone_inkl_Abutment')

    def test_constructor(self):
        with self.assertRaises(FileNotFoundError):
            project = Project(Path('some/bad/path'))
                              
        project = Project(self.projectPath)

    def test_projectFiles(self):
        project = Project(self.projectPath)

        print(project.projectFiles)

    def test_projectOrder(self):
        project = Project(self.projectPath)

        print(project.order)

    def test_untrackedFiles(self):
        project = Project(self.projectPath)
        self.assertTrue(set(project.trackedFiles).isdisjoint(project.untrackedFiles), 'Tracked files cannot contain untracked Files.')
        self.assertTrue(set(project.trackedFiles).issubset(project.filesWithinProjectFolder), 'Tracked files are part of the files of the project.')
        self.assertTrue(set(project.untrackedFiles).issubset(project.filesWithinProjectFolder), 'Untracked files are part of the files of the project.')
        self.assertTrue(project.constructionInfoPath not in set(project.untrackedFiles), 'File .constructionInfo should not be part of untracked files.')
        self.assertTrue(project.dentalProjectPath not in set(project.untrackedFiles), 'File .dentalProject should not be part of untracked files.')
        for file in project.untrackedFiles:
            self.assertEqual(file.suffix, 'stl')


class ThreeShapeProject(unittest.TestCase):
    def test_threeshape(self):
        projectPath = Path('samples/3shape/01_Kappe/24-07-23_Kappen_3ox')
        project = Project(projectPath)
        
    def test_implantDirectionPosition(self):
        with tempfile.TemporaryDirectory() as projectFolder:
            projectPath = Path(projectFolder)

            # regular files:
            implantDirectionPositionFile = projectPath / 'ImplantDirectionPosition_0.xml'
            implantDirectionPositionFile.touch()

            # new file pattern
            implantDirectionPositionFile = projectPath / '35098_20251222_1019_Westkuestendental_Doppler_8_ImplantDirectionPosition_0.xml'
            implantDirectionPositionFile.touch()

            implantDirectionPaths = list(projectPath.glob('*ImplantDirectionPosition*.xml'))

            self.assertEqual(2, len(implantDirectionPaths))
        

if __name__ == '__main__':
    unittest.main()


import requests
from .project import Project

import logging
logger = logging.getLogger(__name__)

def uploadOrder(project: Project, host: str, token: str) -> None:
    """
    Uploads an order to a MillConnect server.

    Parameters
    ----------
    project : Project
        The project containing the order data.
    host : str
        The URL of the MillConnect server.
    token : str
        A valid authentication token for the MillConnect server.

    Returns
    -------
    None

    See Also
    --------
    millconnect.backend.Project
    """
    orderData = {
        'projectName': project.order['projectName'],
        'deliveryType': project.order['deliveryType'],
        'comment': project.order['comment'],
        'creationMethod': 'client',
        'fastline': project.order['fastline'],
        'alternativeDeliveryAddress': project.order['alternativeDeliveryAddress'] if project.order['alternativeDeliveryAddress'] else ''
    }

    files = {}

    if project.dentalProjectPath:
        files['dentalProject'] = project.dentalProjectPath.open('rb')

    if project.constructionInfoPath:
        files['constructionInfo'] = project.constructionInfoPath.open('rb')

    if project.sceneFilePresent and project.order['isAIO']:
        files['sceneFile'] = project.sceneFilePath.open('rb')
        if project.lowerJaw:
            files['lowerJaw'] = project.lowerJaw.open('rb')
        if project.upperJaw:
            files['upperJaw'] = project.upperJaw.open('rb')

    if project.modelInfoPath:
        files['modelInfo'] = project.modelInfoPath.open('rb')

    if project.partialInfoPath:
        files['partialInfo'] = project.partialInfoPath.open('rb')

    if project.scanInfoPath:
        files['scanInfo'] = project.scanInfoPath.open('rb')

    if project.threeshapePath:
        files['threeshape'] = project.threeshapePath.open('rb')

        # add point files compressed as zip
        pointFiles = project.getPoints()
        pointFiles += project.getImplantDirectionFiles()
        if len(pointFiles) > 0:
            compressedPoints = project.compressFiles(pointFiles)
            files['threeshapePoints'] = compressedPoints

    headers = {
        'Authorization': f'Token {token}'
    }
    response = requests.post(f'{host}/orders/', data=orderData, files=files, headers=headers)
    response.raise_for_status()
    newOrder = response.json()

    for element in project.order['elements']:
        elementData = {
            'type': element['type'] if element['type'] else '',
            'material': element['material'],
            'color': element['color'],
            'height': element['height'],
            'comment': element['comment'],
        }
        files = {
            'file': project.getFile(element['file']).open('rb')
        }
        response = requests.post(f'{host}/orders/{newOrder['id']}/elements/', files=files, data=elementData, headers=headers)
        response.raise_for_status()
        newElement = response.json()

        for amenity in element['amenities']:
            amenityData = {
                'element': newElement['id'],
                'amenity': amenity
            }
            response = requests.post(f'{host}/orders/{newOrder['id']}/elements/{newElement['id']}/amenities/', data=amenityData, headers=headers)

        for tooth in element['teeth']:
            response = requests.post(f'{host}/orders/{newOrder['id']}/elements/{newElement['id']}/teeth/', json=tooth, headers=headers)
            response.raise_for_status()


    # mark the order to be ready
    data = {
        'ready': True
    }
    response = requests.patch(f'{host}/orders/{newOrder['id']}/', json=data, headers=headers)
    response.raise_for_status()


import unittest
import asyncio

import sys
import logging
logging.basicConfig(stream=sys.stdout, level=logging.INFO, format='%(asctime)s %(name)s %(levelname)s: %(message)s')
logger = logging.getLogger(__name__)

from src.millconnect import backend
from src.millconnect.server import listen, getAuthentificationToken

import requests
from pathlib import Path

class BackendTest(unittest.IsolatedAsyncioTestCase):

    def test_availableHost(self):
        availableHost = backend.getAvailableHost()
        self.assertIsNotNone(availableHost)

    async def test_authorization(self):
        async def handleAuthorization(websocket, running):
            token = await getAuthentificationToken(websocket)
            logger.info(f'Got token: {token}')

            username = backend.whoAmI('http://zahnomat-backend:8000', token)
            logger.info(username.json())
            running.set_result(False)

        running = asyncio.get_running_loop().create_future()
        server = await listen('localhost', 8765, lambda x: handleAuthorization(x, running))
        logger.info('Started server')

        await running

    async def test_orderCreation(self):
        async def handleOrderCreation(websocket, running):
            token = await getAuthentificationToken(websocket)
            logger.info(f'Got token: {token}')

            host = 'http://zahnomat-backend:8000'

            orderData = {
                'projectName': 'Test project',
                'deliveryType': 1,
                'comment': 'Some comment',
                'creationMethod': 'millConnect',
                'fastline': True,
                'alternativeDeliveryAddress': None
            }
            headers = {
                'Authorization': f'Token {token}',
            }
            response = requests.post(f'{host}/v1/orders/', json=orderData, headers=headers)

            running.set_result(False)
            self.assertEqual(response.status_code, 201)


        running = asyncio.get_running_loop().create_future()
        server = await listen('localhost', 8765, lambda x: handleOrderCreation(x, running))
        logger.info('Started server')

        await running

    async def test_elementCreation(self):
        async def handleOrderCreation(websocket, running):
            token = await getAuthentificationToken(websocket)
            logger.info(f'Got token: {token}')

            host = 'http://zahnomat-backend:8000'

            orderData = {
                'projectName': 'Test project',
                'deliveryType': 1,
                'comment': 'Some comment',
                'creationMethod': 'millConnect',
                'fastline': True,
                'alternativeDeliveryAddress': None
            }
            headers = {
                'Authorization': f'Token {token}',
            }
            response = requests.post(f'{host}/v1/orders/', json=orderData, headers=headers)
            self.assertEqual(response.status_code, 201)
            newOrder = response.json()

            elementData = {
                'type': 1,
                'material': 1,
                'color': 1,
                'comment': 'some comment',
            }
            files = {
                'file': open('tests/samples/86208/1135-gold_-2018-11-28-36-crown_cad.stl', 'rb')
            }
            response = requests.post(f'{host}/v1/orders/{newOrder['id']}/elements/',files=files, data=elementData, headers=headers)
            self.assertEqual(response.status_code, 201)

            running.set_result(False)

        while True:

            running = asyncio.get_running_loop().create_future()
            server = await listen('localhost', 8765, lambda x: handleOrderCreation(x, running))
            logger.info('Started server')

            try:
                await running
                server.close()
            except KeyboardInterrupt:
                break

    async def test_clientVersion(self):
        async def handleOrderCreation(websocket, running):
            token = await getAuthentificationToken(websocket)
            logger.info(f'Got token: {token}')

            host = 'http://zahnomat-backend:8000'

            version = backend._getVersion(host, token)
            print(version)

            running.set_result(False)

        while True:

            running = asyncio.get_running_loop().create_future()
            server = await listen('localhost', 8765, lambda x: handleOrderCreation(x, running))
            logger.info('Started server')

            try:
                await running
                server.close()
            except KeyboardInterrupt:
                break

    async def test_downloadClient(self):
        async def handleOrderCreation(websocket, running):
            token = await getAuthentificationToken(websocket)
            logger.info(f'Got token: {token}')

            host = 'http://zahnomat-backend:8000'

            currentVersion = '0.0.1rc0'
            newFile = backend.downloadClient(host, token, currentVersion, Path('./'))
            self.assertTrue(newFile.exists())

            newFile.unlink()

            running.set_result(False)

        while True:

            running = asyncio.get_running_loop().create_future()
            server = await listen('localhost', 8765, lambda x: handleOrderCreation(x, running))
            logger.info('Started server')

            try:
                await running
                server.close()
            except KeyboardInterrupt:
                break


import unittest
from unittest import mock

from src.millconnect import updater

class UpdaterTest(unittest.TestCase):

    @mock.patch('os.execv')
    @mock.patch('sys.argv')
    def test_restart_with_spaces(self, argv: mock.Mock, execv: mock.Mock):
        argv.return_value = ['python -m millconnect.gui', r'c:\some path\ with space\folder']

        updater.restartIntoNewApplication()

        for arg in execv.call_args[0][1]:
            self.assertTrue(arg.startswith('"') and arg.endswith('"'))